@@ -5,7 +5,7 @@
 #ifndef APISTRING_H_
 #define APISTRING_H_
 
-#include "mruby.h"
+#include <mruby.h>
 
 /* both functions return a null pointer on failure */
 char *mrdb_strndup(mrb_state *mrb, const char *s, size_t size);
